// WICED buffered WiFi Speed test
/**
(c)2016 Forward Computing and Control Pty. Ltd.
www.forward.com.au
This code may be freely used for both private and commercial use.
Provide this copyright is maintained.
*/

// Using Adafruit Feather WICED board
#include <adafruit_feather.h>
#include "pfodBufferedClient.h"


// You need to modify the WLAN_SSID, WLAN_PASS settings below
// to match your network settings
#define WLAN_SSID       "networkSSID"        // cannot be longer than 32 characters!
#define WLAN_PASS       "password"

// also modify the IP address to suit
IPAddress ip(10, 1, 1, 100);


int ADC_pin = PC3; // WICED, Adafruit WICED use PC3, WICED has odd pin names
const unsigned long baudRate = 115200;  // note this is an unsigned long value

unsigned long uS = 0;
unsigned long loopCounter = 0;
const unsigned long PRINT_RESULTS_COUNT = 1000000;
const unsigned long MAX_PRINT_RESULTS = 10;
unsigned long lastMillis = 0;
unsigned long looptime = 0;
int sampledData = 0;
float dataOut = 0.0;
unsigned long  testCounter = 0;

const int portNo = 23; // What TCP port to listen on for connections.
AdafruitTCPServer server(portNo);
AdafruitTCP client;
pfodBufferedClient bufferedClient;


void setup() {
  Serial.begin(baudRate);
  for (int i = 10; i > 0; i--) { // add a little delay, to give you time to open the serial monitor
    delay(1000);
    Serial.print(" ");
    Serial.print(i);
  }
  Serial.println();
  pinMode(PC3, INPUT_ANALOG); // special pinMode for WICED
  /* Initialise wifi module */
  IPAddress gateway(ip[0], ip[1], ip[2], 1); // set gatway to ... 1
  Serial.print(F("Setting ip to: "));
  Serial.println(ip);
  Serial.print(F("Setting gateway to: "));
  Serial.println(gateway);
  IPAddress subnet(255, 255, 255, 0);
  Feather.config(ip, gateway, subnet);

  while ( !Feather.connected() )  {
    delay(500); // delay between each attempt
    if ( Feather.connect(WLAN_SSID, WLAN_PASS) )  {
      Serial.println("Wifi connected");
    } else {
      Serial.printf("Failed! %s (%d)", Feather.errstr(), Feather.errno());
      Serial.println();
    }
  }
  Feather.printNetwork();
  server.err_actions(true, true);   // Tell the TCP Server to auto print error codes and halt on errors
  server.begin();
  Serial.println("Server started");
  // initialize client
  client = server.available(); // evaluates to false if no connection

}

void loop(void) {
  while ((!client) || (!client.connected())) {
    client = server.available();
    if (client.connected()) {
      Serial.println("Got connection");
      bufferedClient.connect(&client); // sets new io stream to read from and write to
      lastMillis = millis(); 
      loopCounter = 0;
    } else {
      client.stop();
      Serial.println("Waiting for connection");
      delay(1000);
    }
  }

  if (loopCounter == PRINT_RESULTS_COUNT) {
    testCounter++;
    long ms = millis();
    Serial.print("Using buffered Adafruit WICED");
    Serial.print(" ");Serial.print(PRINT_RESULTS_COUNT);Serial.print(" loops in:");
    looptime = ms - lastMillis;
    Serial.print(looptime); Serial.print("mS");
    Serial.print(" -> Sample rate:"); Serial.print(1000.0*((float)PRINT_RESULTS_COUNT) / looptime); Serial.println(" samples/sec");
    Serial.print(" Time left to completion: "); Serial.print(((float)looptime) * (MAX_PRINT_RESULTS-testCounter)/ 1000.0); Serial.println(" sec");
    Serial.println("Example data record:");
    // print a sample of the data record
    sendData(&Serial);
    lastMillis = millis(); // restart time after printing
    loopCounter = 0;
    if (testCounter >= MAX_PRINT_RESULTS) {
      bufferedClient.flush();  // finished flush last record since we are not using stop() which would close TeraTerm window
      // stop here to view results
      while (1) {};
    }
  }

  // get the data and the time
  // replace these lines with your own measurement code
  sampledData = analogRead(ADC_pin);
  uS = micros();

  // prepare and print the data record  time , data <nl>
  // do any data scaling and timestamp preperation here
  dataOut = ((float) sampledData) / 1024.0;
  sendData(&bufferedClient);
  loopCounter++;

}  //end main loop

// change this method to send your data
void sendData(Print * output) {
  output->print(uS);
  output->print(',');
  output->print(dataOut);
  output->println();
}

